@@ -1,14 +1,14 @@
 /*  remind:   reminder program
 
     NAME
-        remind - a reminder program
+    remind - a reminder program
 
     SYNOPSIS
-        remind  [-a] [-c colour_pairs] [-d date] [-D n[,n] ...] [-e]
-                [-f filename] [-h] [-i] [-l] [-L] [-m n[,n] ...] [-p]
-                [-P pointer] [-q] [-r repeat] [-s] [-t timeout]
-                [-u urgency] [-w warning] [-X n[,n] ...]
-                [message]
+    remind  [-a] [-c colour_pairs] [-d date] [-D n[,n] ...] [-e]
+    [-f filename] [-h] [-i] [-l] [-L] [-m n[,n] ...] [-p]
+    [-P pointer] [-q] [-r repeat] [-s] [-t timeout]
+    [-u urgency] [-v] [-w warning] [-X n[,n] ...] [-z]
+    [message]
 
     See remind(1) man page for more.
 */
@@ -44,7 +44,8 @@ enum cmd_type {
     CMD_LIST,
     CMD_LIST_HEADER,
     CMD_MODIFY,
-    CMD_MOD_POINTER
+    CMD_MOD_POINTER,
+    CMD_ZZZ
 };
 
 /* Structure for action number list */
@@ -64,7 +65,9 @@ struct st_params {
     bool quiet;
     char* hilite;
     int pointer;
+    bool version;
     struct st_nlist* actlist;
+    bool done;
 };
 
 typedef struct st_params PARAMS;
@@ -72,13 +75,14 @@ typedef struct st_params PARAMS;
 /* datafile error messages matching error codes */
 char* error_msg[] = {
     "unused",
-    "no such file: %s",
+    "unable to open database file: %s",
     "record [%03d]: bad seek",
     "record [%03d]: bad read",
     "record [%03d]: bad write",
     "action [%03d] does not exist",
-    "unable to create remind file: %s",
-    "that's no remind file: %s",
+    "unable to create database file: %s",
+    "database file does not match current version",
+    "that's no database file: %s",
     "action [%03d] is on free list",
     "action [%03d] can't be found on its list"
 };
@@ -183,22 +187,22 @@ bool parse_repeat(char* rstr, enum repeat_type* type, int* day, int* nday)
     *nday = 1; /* default */
     ntoks = sscanf(rstr,"%c%d,%d",&typech,day,nday);
     switch (typech) {
-        case 'y':
-            *type = RT_YEAR;
-            break;
-        case 'm':
-            *type = RT_MONTH;
-            break;
-        case 'w':
-            *type = RT_WEEK;
-            if (ntoks < 2) return false;
-            break;
-        case 'n':
-            *type = RT_MONTH_WEEK;
-            if (ntoks < 2) return false;
-            break;
-        default:
-            return false;
+    case 'y':
+        *type = RT_YEAR;
+        break;
+    case 'm':
+        *type = RT_MONTH;
+        break;
+    case 'w':
+        *type = RT_WEEK;
+        if (ntoks < 2) return false;
+        break;
+    case 'n':
+        *type = RT_MONTH_WEEK;
+        if (ntoks < 2) return false;
+        break;
+    default:
+        return false;
     }
     return true;
 }
@@ -209,7 +213,7 @@ bool parse_cmd_args(int argc, char *argv[], PARAMS* params, ACTREC* newact)
 {
     int nargs;
     bool first_word = true;
-    char *s, *np, *c, *term;
+    char *s;
     char *switcharg = "dwuDmxtfcPXr"; /* switches that have arguments */
 
     /* set effective time? */
@@ -218,12 +222,14 @@ bool parse_cmd_args(int argc, char *argv[], PARAMS* params, ACTREC* newact)
     /* set defaults */
     params->cmd = CMD_DISPLAY;
     params->quiet = false;
+    params->version = false;
     params->urgency = -1;
     params->colour_set = false;
     params->set_type = ACT_PERIODIC|ACT_STANDARD;
     params->hilite = "";
     params->actlist = NULL;
     params->filename = getenv(REMIND_ENV);
+    params->done = false;
     if (params->filename == NULL) params->filename = REMIND_FILE;
     for (int i=0;i<URGCOL;i++) params->ucol[i] = ((i%2==0)?37:40);
 
@@ -236,6 +242,7 @@ bool parse_cmd_args(int argc, char *argv[], PARAMS* params, ACTREC* newact)
     newact->repeat.type = EOF;
     newact->repeat.day = 0;
     newact->repeat.nday = 0;
+    newact->next_event = 0;
 
     while (--argc > 0 && (*++argv)[0] == '-') {
         for (s=argv[0]+1;*s != '\0'; s++) {
@@ -243,110 +250,116 @@ bool parse_cmd_args(int argc, char *argv[], PARAMS* params, ACTREC* newact)
                 error(ABORT,"switch -%c requires argument",*s);
             }
             switch (*s) {
-                case 'a':
-                    params->set_type = ACT_PERIODIC|ACT_STANDARD;
-                    break;
-                case 'c':
-                    for (int i=0;i<URGCOL;i+=2) {
-                        nargs = sscanf(*++argv,"%d,%d",&(params->ucol[i]),
-                                       &(params->ucol[i+1]));
-                        --argc;
-                        if (nargs == EOF || nargs != 2)
-                            error(ABORT,"bad colour pair");
-                    }
-                    params->colour_set = true;
-                    break;
-                case 'd':
-                    if ((newact->time = date_parse(*++argv,TIME_EOD)) <= 0) {
-                        error(ABORT,"bad date format");
-                    }
-                    if (params->set_type != ACT_STANDARD)
-                        newact->type = ACT_PERIODIC;
+            case 'a':
+                params->set_type = ACT_PERIODIC|ACT_STANDARD;
+                break;
+            case 'c':
+                for (int i=0;i<URGCOL;i+=2) {
+                    nargs = sscanf(*++argv,"%d,%d",&(params->ucol[i]),
+                                   &(params->ucol[i+1]));
                     --argc;
-                    break;
-                case 'D':
-                    params->actlist = parse_int_list(*++argv);
-                    if (!params->actlist) error(ABORT,"bad message list");
-                    params->cmd = CMD_DELETE;
-                    --argc;
-                    break;
-                case 'e':
-                    params->cmd = CMD_EXPORT;
-                    break;
-                case 'f':
-                    params->filename = *++argv;
-                    --argc;
-                    break;
-                case 'h':
-                    /* set highlight off sequence */
-                    params->hilite = "\033[0m";
-                    break;
-                case 'i':
-                    params->cmd = CMD_INIT;
-                    break;
-                case 'l':
-                    params->cmd = CMD_LIST;
-                    break;
-                case 'L':
-                    params->cmd = CMD_LIST_HEADER;
-                    break;
-                case 'm':
-                    params->actlist = parse_int_list(*++argv);
-                    if (!params->actlist) error(ABORT,"bad message list");
-                    params->cmd = CMD_MODIFY;
-                    params->quiet = true; /* silence on redefined action */
-                    --argc;
-                    break;
-                case 'p':
+                    if (nargs == EOF || nargs != 2)
+                        error(ABORT,"bad colour pair");
+                }
+                params->colour_set = true;
+                break;
+            case 'd':
+                if ((newact->time = date_parse(*++argv,TIME_EOD)) <= 0) {
+                    error(ABORT,"bad date format");
+                }
+                if (params->set_type != ACT_STANDARD)
                     newact->type = ACT_PERIODIC;
-                    params->set_type = ACT_PERIODIC;
-                    break;
-                case 'P':
-                    params->pointer = atoi(*++argv);
-                    params->cmd = CMD_MOD_POINTER;
-                    --argc;
-                    break;
-                case 'q':
-                    params->quiet = true;
-                    break;
-                case 'r':
-                    if (!parse_repeat(*++argv,&(newact->repeat.type),
-                                      &(newact->repeat.day),
-                                      &(newact->repeat.nday)))
-                        error(ABORT,"bad repeat option");
-                    --argc;
-                    newact->type = ACT_PERIODIC;
-                    break;
-                case 's':
-                    newact->type = ACT_STANDARD;
-                    params->set_type = ACT_STANDARD;
-                    break;
-                case 't':
-                    newact->timeout = atoi(*++argv);
-                    if (newact->timeout < 0) error(ABORT,"bad timeout value");
-                    --argc;
-                    break;
-                case 'u':
-                    params->urgency = atoi(*++argv);
-                    if (params->urgency < 0 || params->urgency > 4) {
-                        error(ABORT,"bad urgency value");
-                    }
-                    newact->urgency = params->urgency;
-                    --argc;
-                    break;
-                case 'w':
-                    newact->warning = atoi(*++argv);
-                    if (newact->warning < 0 ) error(ABORT,"bad warning value");
-                    --argc;
-                    break;
-                case 'X':
-                    params->actlist = parse_int_list(*++argv);
-                    if (!params->actlist) error(ABORT,"bad dump list");
-                    --argc;
-                    params->cmd = CMD_DUMP;
-                    break;
-                default:
-                    error(ABORT,"illegal option \"-%c\"\n",*s);
+                --argc;
+                break;
+            case 'D':
+                params->actlist = parse_int_list(*++argv);
+                if (!params->actlist) error(ABORT,"bad message list");
+                params->cmd = CMD_DELETE;
+                --argc;
+                break;
+            case 'e':
+                params->cmd = CMD_EXPORT;
+                break;
+            case 'f':
+                params->filename = *++argv;
+                --argc;
+                break;
+            case 'h':
+                /* set highlight off sequence */
+                params->hilite = "\033[0m";
+                break;
+            case 'i':
+                params->cmd = CMD_INIT;
+                break;
+            case 'l':
+                params->cmd = CMD_LIST;
+                break;
+            case 'L':
+                params->cmd = CMD_LIST_HEADER;
+                break;
+            case 'm':
+                params->actlist = parse_int_list(*++argv);
+                if (!params->actlist) error(ABORT,"bad message list");
+                params->cmd = CMD_MODIFY;
+                params->quiet = true; /* silence on redefined action */
+                --argc;
+                break;
+            case 'p':
+                newact->type = ACT_PERIODIC;
+                params->set_type = ACT_PERIODIC;
+                break;
+            case 'P':
+                params->pointer = atoi(*++argv);
+                params->cmd = CMD_MOD_POINTER;
+                --argc;
+                break;
+            case 'q':
+                params->quiet = true;
+                break;
+            case 'r':
+                if (!parse_repeat(*++argv,&(newact->repeat.type),
+                                  &(newact->repeat.day),
+                                  &(newact->repeat.nday)))
+                    error(ABORT,"bad repeat option");
+                --argc;
+                newact->type = ACT_PERIODIC;
+                break;
+            case 's':
+                newact->type = ACT_STANDARD;
+                params->set_type = ACT_STANDARD;
+                break;
+            case 't':
+                newact->timeout = atoi(*++argv);
+                if (newact->timeout < 0) error(ABORT,"bad timeout value");
+                --argc;
+                break;
+            case 'u':
+                params->urgency = atoi(*++argv);
+                if (params->urgency < 0 || params->urgency > 4) {
+                    error(ABORT,"bad urgency value");
+                }
+                newact->urgency = params->urgency;
+                --argc;
+                break;
+            case 'v':
+                params->version = true;
+                break;
+            case 'w':
+                newact->warning = atoi(*++argv);
+                if (newact->warning < 0 ) error(ABORT,"bad warning value");
+                --argc;
+                break;
+            case 'X':
+                params->actlist = parse_int_list(*++argv);
+                if (!params->actlist) error(ABORT,"bad dump list");
+                --argc;
+                params->cmd = CMD_DUMP;
+                break;
+            case 'z':
+                params->cmd = CMD_ZZZ;
+                break;
+            default:
+                error(ABORT,"illegal option \"-%c\"\n",*s);
             }
         }
     }
@@ -385,7 +398,7 @@ bool parse_cmd_args(int argc, char *argv[], PARAMS* params, ACTREC* newact)
     if (params->cmd != CMD_MODIFY) {
         if (newact->timeout < 0) newact->timeout = 0;
         if (newact->urgency < 0 )newact->urgency = 4;
-        if (newact->warning < 0 ) newact->warning = 5;
+        if (newact->warning < 0 ) newact->warning = 7;
         if (newact->repeat.type == EOF) newact->repeat.type = RT_YEAR;
         if (newact->time == 0) newact->time = date_now_eod();
     }
@@ -413,58 +426,57 @@ struct tm* mw_ev_time(time_t* now_time, struct st_repeat repeat, int month)
 }
 
 
-time_t make_active_time(ACTREC* action)
+time_t make_active_time(ACTREC* action, time_t base_time)
 {
     double delta;
-    int day_diff, now_wday, now_mon, period, now_mday;
-    struct tm* ev_tm, *now;
+    int now_mon, period, now_mday;
+    struct tm* ev_tm;
 
-    time_t event_time, nowtime, fd_time, act_time;
+    time_t event_time;
 
-    nowtime = date_now();
     switch (action->repeat.type) {
-        case RT_YEAR:
-            event_time = date_make_current(action->time,YEAR_ONLY);
-            break;
-        case RT_MONTH:
-            event_time = date_make_current(action->time,YEAR_AND_MONTH);
-            break;
-        case RT_WEEK:
-            delta = difftime(nowtime,action->time);
-            if (delta < 0) {
-                /* action time is in the future; just return it */
-                event_time = action->time;
+    case RT_YEAR:
+        event_time = date_make_current(action->time,YEAR_ONLY, base_time);
+        break;
+    case RT_MONTH:
+        event_time = date_make_current(action->time,YEAR_AND_MONTH, base_time);
+        break;
+    case RT_WEEK:
+        delta = difftime(base_time,action->time);
+        if (delta < 0) {
+            /* action time is in the future; just return it */
+            event_time = action->time;
+        }
+        else {
+            /* compute next event time */
+            period = (action->repeat.nday==0?1:action->repeat.nday) * 7 *
+                SECSPERDAY;
+            delta = ceil(delta / SECSPERDAY) * SECSPERDAY;
+            event_time = base_time + period - ((int) delta)%period;
+        }
+        break;
+    case RT_MONTH_WEEK:
+        ev_tm = localtime(&base_time);
+        now_mon = ev_tm->tm_mon;
+        now_mday = ev_tm->tm_mday;
+        ev_tm = mw_ev_time(&base_time,action->repeat,now_mon);
+        if (ev_tm->tm_mon > now_mon) {
+            /* calculated event day next month, look for last wday
+             * in current month */
+            while (ev_tm->tm_mon > now_mon) {
+                ev_tm->tm_mday -= 7;
+                event_time = mktime(ev_tm);
             }
-            else {
-                /* compute next event time */
-                period = (action->repeat.nday==0?1:action->repeat.nday) * 7 *
-                    SECSPERDAY;
-                delta = ceil(delta / SECSPERDAY) * SECSPERDAY;
-                event_time = nowtime + period - ((int) delta)%period;
-            }
-            break;
-        case RT_MONTH_WEEK:
-            ev_tm = localtime(&nowtime);
-            now_mon = ev_tm->tm_mon;
-            now_mday = ev_tm->tm_mday;
-            ev_tm = mw_ev_time(&nowtime,action->repeat,now_mon);
-            if (ev_tm->tm_mon > now_mon) {
-                /* calculated event day next month, look for last wday
-                 * in current month */
-                while (ev_tm->tm_mon > now_mon) {
-                    ev_tm->tm_mday -= 7;
-                    event_time = mktime(ev_tm);
-                }
-            }
-            else if (ev_tm->tm_mday < now_mday) {
-                /* current day is later than this month's occurrence;
-                 * check next month */
-                ev_tm = mw_ev_time(&nowtime,action->repeat,now_mon+1);
-            }
-            event_time = mktime(ev_tm);
-            break;
-        default:
-            error(ABORT,"invalid repeat type found: %d",action->repeat.type);
+        }
+        else if (ev_tm->tm_mday < now_mday) {
+            /* current day is later than this month's occurrence;
+             * check next month */
+            ev_tm = mw_ev_time(&base_time,action->repeat,now_mon+1);
+        }
+        event_time = mktime(ev_tm);
+        break;
+    default:
+        error(ABORT,"invalid repeat type found: %d",action->repeat.type);
     }
     return event_time;
 }
@@ -497,7 +509,7 @@ void display(ACTYPE type, int urgency, bool quiet, char* hilite)
         if (action->timeout != 0 &&
             difftime(date_now(),action->time) >
             (action->timeout-1) * SECSPERDAY) {
-            if (act_delete(actno) != 0)
+            if (act_delete(actno, true) != 0)
                 error(ABORT,error_msg[rem_error()], actno);
         }
         else if (urgency < 0 && action->urgency == 0) {
@@ -505,37 +517,48 @@ void display(ACTYPE type, int urgency, bool quiet, char* hilite)
         }
         else {
             switch (type) {
-                case ACT_STANDARD:
-                    if ((urgency < 0 ||
-                         (urgency >= 0 && urgency == action->urgency)) &&
-                        difftime(date_now(),action->time) > -SECSPERDAY) {
-                        if (action->urgency == 0) action->urgency = 4;
-                        printf("%s[%03d] %s%s\n",
-                               hilite_on(action->urgency,hilite), actno,
-                               action->msg,hilite);
+            case ACT_STANDARD:
+                if ((urgency < 0 ||
+                     (urgency >= 0 && urgency == action->urgency)) &&
+                    difftime(date_now(),action->time) > -SECSPERDAY) {
+                    if (action->urgency == 0) action->urgency = 4;
+                    printf("%s[%03d] %s%s\n",
+                           hilite_on(action->urgency,hilite), actno,
+                           action->msg,hilite);
+                }
+                break;
+            case ACT_PERIODIC:
+                event_time = make_active_time(action, date_now());
+                delta = difftime(event_time,date_now());
+                /* Snoozed event reset? */
+                if (action->next_event) {
+                    double ndelta = difftime(action->next_event, date_now());
+                    if (ndelta <= (action->warning+1)*SECSPERDAY) {
+                        action->next_event = 0;
+                        if (act_write(actno, action) != 0) {
+                            error(ABORT,"unable to update action: %d", actno);
+                        }
                     }
-                    break;
-                case ACT_PERIODIC:
-                    event_time = make_active_time(action);
-                    delta = difftime(event_time,date_now());
-                    if (delta > 0 && delta <= (action->warning+1)*SECSPERDAY &&
-                        (urgency < 0 ||
-                         (urgency >= 0 && urgency == action->urgency))) {
-                        delta_days = floor(delta/SECSPERDAY);
-                        printf("%s[%03d] [%s]",
-                               hilite_on(action->urgency, hilite),
-                               actno, date_str(event_time));
-                        if (delta_days == 1)
-                            printf(" (tomorrow) ");
-                        else if (delta_days == 0)
-                            printf(" (today) ");
-                        else
-                            printf(" (%2d days) ",delta_days);
-                        printf("%s%s\n",action->msg,hilite);
-                    }
-                    break;
-                default:
-                    error(ABORT,"bad action type: %d",type);
+                }
+                if (delta >= 0 && delta <= (action->warning+1)*SECSPERDAY &&
+                    !action->next_event &&
+                    (urgency < 0 ||
+                     (urgency >= 0 && urgency == action->urgency))) {
+                    delta_days = floor(delta/SECSPERDAY);
+                    printf("%s[%03d] [%s]",
+                           hilite_on(action->urgency, hilite),
+                           actno, date_str(event_time));
+                    if (delta_days == 1)
+                        printf(" (tomorrow) ");
+                    else if (delta_days == 0)
+                        printf(" (today) ");
+                    else
+                        printf(" (%2d days) ",delta_days);
+                    printf("%s%s\n",action->msg,hilite);
+                }
+                break;
+            default:
+                error(ABORT,"bad action type: %d",type);
             }
         }
         actno = act_iter_next();
@@ -544,9 +567,9 @@ void display(ACTYPE type, int urgency, bool quiet, char* hilite)
         char *typestr = (type==ACT_STANDARD?"standard":"periodic");
         if (nhidden == 1)
             printf(">>>>> There is one background %s action\n",typestr);
-       else
-           printf(">>>>> There are %d background %s actions\n",nhidden,
-                  typestr);
+        else
+            printf(">>>>> There are %d background %s actions\n",nhidden,
+                   typestr);
     }
     return;
 }
@@ -568,13 +591,14 @@ void dump_action(int actno)
 
     if ((action = act_read(actno)) != NULL) {
         printf("--Action: %d--\n",actno);
-        printf("Type:    %d\n",action->type);
+        printf("Type:    %s\n",str_act_type(action->type));
         printf("Next:    %d\n",action->next);
         printf("Urgency: %d\n",action->urgency);
         printf("Warning: %d\n",action->warning);
         printf("Date:    %s\n",date_str(action->time));
         printf("Repeat:  %s\n",repeat_str(action->repeat));
         printf("Timeout: %d\n",action->timeout);
+        printf("NextEvt: %s\n",date_full_str(action->next_event));
         printf("Msg:     \"%s\"\n",action->msg);
     }
     else {
@@ -583,9 +607,10 @@ void dump_action(int actno)
     return;
 }
 
-void delete_action(int actno)
+void delete_action(int actno, bool nullify)
 {
-    if ((act_delete(actno) != 0)) error(CONTINUE,error_msg[rem_error()],actno);
+    if ((act_delete(actno, nullify) != 0))
+        error(CONTINUE, error_msg[rem_error()], actno);
     return;
 }
 
@@ -660,7 +685,7 @@ void modify_action(int actno,ACTREC* newact)
     if ((newact->urgency >= 0 && save.type == ACT_STANDARD) ||
         (newact->time > 0 && save.type == ACT_PERIODIC)) {
         /* need to re-insert action into list; potential sequence change */
-        delete_action(actno);
+        delete_action(actno, false);
         define_action(&save,true);
     }
     else {
@@ -679,12 +704,16 @@ void modify_action_pointer(int actno, int pointer)
     act_write(actno, action);
 }
 
+enum {
+    DAYMONSTRLEN = 5
+};
+
 void export(char* filename)
 {
     ACTREC* action;
     REMHDR* header;
     int actno;
-
+    char daymon[DAYMONSTRLEN+1];
 
     if ((header = rem_header()) == NULL) error(ABORT,error_msg[rem_error()]);
 
@@ -709,13 +738,35 @@ void export(char* filename)
     while (actno) {
         if ((action = act_read(actno)) == NULL)
             error(ABORT,error_msg[rem_error()],actno);
+        strncpy(daymon, date_str(action->time), DAYMONSTRLEN);
+        daymon[DAYMONSTRLEN] = '\0';
         printf("remind -fruwtdq %s %s %d %d %d %s \"%s\"\n", filename,
                repeat_str(action->repeat),  action->urgency, action->warning,
-               action->timeout, date_str(action->time), action->msg);
+               action->timeout, daymon, action->msg);
         actno = act_iter_next();
     }
 }
 
+void toggle_next_event_time(int actno)
+{
+    ACTREC* action;
+    time_t event_time, next_event_time;
+
+    action = act_read(actno);
+    if (action->next_event) {
+        action->next_event = 0;
+    }
+    else {
+        event_time = make_active_time(action, date_now());
+        next_event_time = make_active_time(action, event_time + SECSPERDAY);
+        action->next_event = next_event_time;
+    }
+    if (act_write(actno, action) != 0) {
+        error(ABORT,"unable to update action: %d", actno);
+    }
+    return;
+}
+
 bool perform_cmd(PARAMS* params, ACTREC* newact)
 {
     struct st_nlist* actno;
@@ -725,61 +776,72 @@ bool perform_cmd(PARAMS* params, ACTREC* newact)
         error(ABORT,error_msg[rem_error()],params->filename);
     if (params->colour_set) rem_set_hilite(params->ucol);
 
+    if (params->version) {
+        printf("remind %s\n",GIT_VERSION);
+    }
+
     switch (params->cmd) {
-        case CMD_DEFINE:
-            define_action(newact,params->quiet);
-            break;
-        case CMD_DELETE:
-            actno = params->actlist;
-            while (actno != NULL) {
-                delete_action(actno->n);
-                actno = actno->next;
-            }
-            break;
-        case CMD_DISPLAY:
-            if (params->set_type & ACT_PERIODIC) {
-                display(ACT_PERIODIC, params->urgency, params->quiet,
-                        params->hilite);
-            }
-            if (params->set_type & ACT_STANDARD) {
-                display(ACT_STANDARD, params->urgency, params->quiet,
-                        params->hilite);
-            }
-            break;
-        case CMD_DUMP:
-            actno = params->actlist;
-            while (actno != NULL) {
-                dump_action(actno->n);
-                actno = actno->next;
-            }
-            break;
-        case CMD_EXPORT:
-            export(params->filename);
-            break;
-        case CMD_INIT:
-            create_file(params->filename, params->ucol,
-                               params->quiet);
-            break;
-        case CMD_LIST:
-        case CMD_LIST_HEADER:
-            list_actions(params->cmd,params->set_type);
-            break;
-        case CMD_MODIFY:
-            actno = params->actlist;
-            while (actno != NULL) {
-                modify_action(actno->n,newact);
-                actno = actno->next;
-            }
-            break;
-        case CMD_MOD_POINTER:
-            actno = params->actlist;
-            while (actno != NULL) {
-                modify_action_pointer(actno->n,params->pointer);
-                actno = actno->next;
-            }
-            break;
-        default:
-            error(ABORT,"internal command error: %d",params->cmd);
+    case CMD_DEFINE:
+        define_action(newact,params->quiet);
+        break;
+    case CMD_DELETE:
+        actno = params->actlist;
+        while (actno != NULL) {
+            delete_action(actno->n, params->quiet);
+            actno = actno->next;
+        }
+        break;
+    case CMD_DISPLAY:
+        if (params->set_type & ACT_PERIODIC) {
+            display(ACT_PERIODIC, params->urgency, params->quiet,
+                    params->hilite);
+        }
+        if (params->set_type & ACT_STANDARD) {
+            display(ACT_STANDARD, params->urgency, params->quiet,
+                    params->hilite);
+        }
+        break;
+    case CMD_DUMP:
+        actno = params->actlist;
+        while (actno != NULL) {
+            dump_action(actno->n);
+            actno = actno->next;
+        }
+        break;
+    case CMD_EXPORT:
+        export(params->filename);
+        break;
+    case CMD_INIT:
+        create_file(params->filename, params->ucol,
+                    params->quiet);
+        break;
+    case CMD_LIST:
+    case CMD_LIST_HEADER:
+        list_actions(params->cmd,params->set_type);
+        break;
+    case CMD_MODIFY:
+        actno = params->actlist;
+        while (actno != NULL) {
+            modify_action(actno->n,newact);
+            actno = actno->next;
+        }
+        break;
+    case CMD_MOD_POINTER:
+        actno = params->actlist;
+        while (actno != NULL) {
+            modify_action_pointer(actno->n,params->pointer);
+            actno = actno->next;
+        }
+        break;
+    case CMD_ZZZ:
+        actno = params->actlist;
+        while (actno != NULL) {
+            toggle_next_event_time(actno->n);
+            actno = actno->next;
+        }
+        break;
+    default:
+        error(ABORT,"internal command error: %d",params->cmd);
     }
     if (rem_cls() == EOF) error(ABORT,"close failed: %s",params->filename);
     if ((errno = rem_error()) != 0) error(ABORT,error_msg[errno],0);
